% Analyse Raw configuration - 09/30/11
% Jean-Marc Berthommé
%
% - 09/30/11:
%   . file to analyse all the results of the 'Raw' configuration

function analyse_raw
opt = 'Raw';               % configuration
dopt = 'KLD';              % distance option
FirstIm = 192;             % first image
LastIm  = 309;             % last  image
N = 50;                    % tolerance resolution with ground truth
vmax = 10;                 % nb ob test version to integrate
tol_px = linspace(0,60,N); % levels of tolerance in pixels
tol_pct = linspace(0,1,N); % levels of tolerance in percentage
scr = get(0,'ScreenSize'); % screen size
FileGT = 'GT_WBS1C.txt';   % 'WalkByShop1cor' ground truth

fi = fopen(FileGT,'r');
GT = importValues(fi, FirstIm, LastIm); % ground truth

Tc = zeros(vmax,N); % array collecting the okC on 10 versions
Ta = zeros(vmax,N); % array collecting the okA on 10 versions

for cut = 1:3:7 % 1:3:10
    for i = 1:vmax
        FileAT = sprintf('%s_%s_%d_v%d.txt', opt, dopt, cut, i);
        fi = fopen(FileAT,'r');
        AT = importValues(fi, FirstIm, LastIm); % algorithm truth

        okC = trk_center_err(GT, AT, tol_px);   % track center errors
        okA = trk_area_err(GT, AT, tol_pct);    % track area   errors
        Tc(i,:) = okC; Ta(i,:) = okA;           % record error version
        % disp_track_rates(scr, tol_px,okC, tol_pct,okA, 'r',1,'-');
    end
    
    tkCol = [0.1*cut 1-0.1*cut 0.35]; % track color: ~green -> ~red
    
    avgOkC = mean(Tc,1); avgOkA = mean(Ta,1);
    minOkC = min(Tc,[],1); minOkA = min(Ta,[],1);
    maxOkC = max(Tc,[],1);  maxOkA = max(Ta,[],1);
    
    disp_track_rates(scr, tol_px, avgOkC, tol_pct, avgOkA, tkCol,2,'-');
    disp_track_rates(scr, tol_px, minOkC, tol_pct, minOkA, tkCol,1,'-.');
    disp_track_rates(scr, tol_px, maxOkC, tol_pct, maxOkA, tkCol,1,'-.');
end

export_fig;

fprintf('Press any key to continue...\n');
pause; close all; clear all;

function GT = importValues(fi, FirstImage, LastImage)
GT = [];
for i=FirstImage:LastImage
    [nIm x1 x2 y1 y2 Width Height cut xc yc] = getValues(fi);
    gt = [nIm x1 x2 y1 y2 Width Height cut xc yc];
    GT = cat(1, GT, gt);
end
fclose(fi);

function [nIm x1 x2 y1 y2 Width Height cut xc yc] = getValues(fid)
fread(fid, 9, '*char');
nIm = fscanf(fid, '%d');
fread(fid, 16, '*char');
x1 = fscanf(fid, '%d');
fread(fid, 1, '*char');
x2 = fscanf(fid, '%d');
fread(fid, 9, '*char');
y1 = fscanf(fid, '%d');
fread(fid, 1, '*char');
y2 = fscanf(fid, '%d');
fread(fid, 14, '*char');
Width = fscanf(fid, '%d');
fread(fid, 13, '*char');
Height = fscanf(fid, '%d');
fread(fid, 10, '*char');
cut = fscanf(fid, '%d');
fread(fid, 10, '*char');
xc = fscanf(fid, '%d');
fread(fid, 9, '*char');
yc = fscanf(fid, '%d');

function okC = trk_center_err(GT, AT, tol_px)
% Euclidian distance between the centers of the ROIs of GT and AT
% set of Euclidian errors between the ROI centers along the track (in px)
Er = ((GT(:,9)-AT(:,9)).^2 + (GT(:,10)-AT(:,10)).^2).^(1/2);

% matrix of "OKs" along the track in function of the tolerance
ok = (Er * ones(size(tol_px)) <= ones(size(Er)) * tol_px);
okC = mean(ok,1);

function okA = trk_area_err(GT, AT, tol_pct)
% "%" of common area between the ROIs of GT and AT
% "inter" areas between the GT and AT ROIs
xi1 = max(GT(:,2), AT(:,2)); xi2 = min(GT(:,3), AT(:,3));
yi1 = max(GT(:,4), AT(:,4)); yi2 = min(GT(:,5), AT(:,5));
A_int = zeros(size(GT,1),1); t = (xi1 < xi2) & (yi1 < yi2); % test
A_int(t) = (xi2(t)-xi1(t)+1).*(yi2(t)-yi1(t)+1); A_int(~t) = 0;

% "union" areas between the GT and AT ROIs
A_gt = (GT(:,3) - GT(:,2) + 1).*(GT(:,5) - GT(:,4) + 1);
A_tt = (AT(:,3) - AT(:,2) + 1).*(AT(:,5) - AT(:,4) + 1);
A_uni = A_gt + A_tt - A_int;

% set of % of common area over the unified area between GT and AT
ErA = 1 - (A_int./A_uni);

% matrix of "OKs" along the track in function of the tolerance
okA = (ErA * ones(size(tol_pct)) <= ones(size(ErA)) * tol_pct);
okA = mean(okA,1);

function disp_track_rates(scr, tol_px,okC, tol_pct,okA, col,width,style)

f1 = figure(1); set(gcf,'Color',[0.2,0.2,0.2]);
set(f1,'Position',[2*scr(3)/3 scr(4)/2 scr(3)/3 scr(4)/2-100]);   
plot(tol_px,okC,'color',col,'linewidth',width,'linestyle',style); hold on;
% xlabel('tolerance [px]','color','w');
% ylabel('taux de bon suivi [%]','color','w');
% tit = sprintf('%% de bon suivi = f(tol)');% cut = %d - v%d',cut,i);
% title(tit,'color','r');

f2 = figure(2); set(gcf,'Color',[0.2,0.2,0.2]);
set(f2,'Position',[2*scr(3)/3 0 scr(3)/3 scr(4)/2-100]);
plot(tol_pct,okA,'color',col,'linewidth',width,'linestyle',style); hold on;
% xlabel('tolerance [% d''aire]','color','w');
% ylabel('taux de bon suivi [%]','color','w');
% tit = sprintf('%% de bon suivi = f(tol)');% cut = %d - v%d',cut,i);
% title(tit,'color','r');

function export_fig
print(1, 'center_Raw.eps', '-depsc', '-cmyk');
print(2, 'area_Raw.eps', '-depsc', '-cmyk');

