% Kullback-Leibler divergence - 09/26/11
% Jean-Marc Berthommé
% 
% - 02/15/10:
%   . 1st version based on "High dimensional statistical measure for 
%     region-of-interest tracking" by by Sylvain Boltz et al.
%   . estimates the Kullback-Leibler divergence between a Reference and a 
%     Target pdf, R and T here, through a kNN framework
% - 09/26/11 :
%   . faster version

function Dkl = D_KL(T, R, k)
% R : pdf of the Reference patch - nr x dr - nr = hr x wr
% T : pdf of the Target    patch - nt x dt - nt = ht x wt
% k  : nearest neighbour          -  1 x 1

[nr, dr] = size(R); [nt, dt] = size(T);
if (dr ~= dt), error('Dimension mismatch between pdfs'); end;

if isequal(R, T)
    Dkl = log(nr/(nt-1)); % avoid calculations if equal
else
    dtr = knn_search(T, R, k);  % Target in Reference knn distances
    dtt = knn_search(T, T, k);  % Target in Target    knn distances
    
    logR = zeros(nt,1);
    ok = (dtr ~=0) & (dtt ~=0);
    logR(ok) = log(dtr(ok)./dtt(ok)); logR(~ok) = 0;
    sumLogR = sum(logR);
    
    Dkl = log(nr/(nt-1)) + (dr/nt)*sumLogR;
end