% kNN Tracker - 10/18/11
% Jean-Marc Berthommé
%
% - 02/15/10:
%   . 1st version from T. Chateau MCMC PF tracker
%   . kNN tracking based on a XYRGB representation
% - 01/03/11:
%   . formal bugs removed
% - 01/20/11:
%   . new optional models: Classical, Gaussian and Random (gaussian)
% - 09/13/11:
%   . new formatting
%   . 'im1' 192-309 & 630-800 - 'parc' 60-75 & 102-114 & 154-169 & 258-278
%   . average smoothing option (Avg) added to buildModel
% - 09/14/11:
%   . new 'Ker' mode for kernels
% - 09/17/11:
%   . display particules in function of their distance to the reference
%   . 'Ker' works now for SAD/SSD as well
% - 09/18/11:
%   . new modes: RndRaw/RndGauss/RndVor// and Ker -> KerOpt
%   . Map fixed for RndRaw/RndGauss/RndVor
% - 10/18/11:
%   . WalkByShop1cor 192-309

function XYRGB_knn_tracker %(FirstIm, LastIm, cut, opt, dopt, K)
FirstIm = 192;             % first image number
LastIm  = 194;             % last  image number -> replace 194 by 309 as soon
% as you have downloaded the WalkByShop1cor.tar.gz sequence at:
% http://homepages.inf.ed.ac.uk/rbf/CAVIARDATA1/
cut = 4;                   % step between 2 pixels: 1/4/7/10
opt = 'Raw';               % Raw/Avg/Gauss/RndRaw/RndGauss/RndVor/KerOpt
dopt = 'KLD';              % distance option: KLD/SAD/SSD/KMI
K = 5;                     % nb of kernels to select if opt = 'KerOpt'

scr = get(0,'ScreenSize'); % screen size
Map = [];                  % Map used for RndRaw/RndGauss/RndVor
kNN = 3;                   % 'ith' nearest neighbor
Ker = [];                  % kernels to select
Wker = [];                 % kernels relative weights
if strcmp('KerOpt',opt)
    cut = 1;               % cut forced to 1 for 'KerOpt'
    load('KerOpt10.mat');
    Wker = disp_kernels(scr, Ker, K);
end

nam = 'WalkByShop1cor';           % file name
ext = 'jpg';                      % extension used
debug = 1;                        % release/debug flag - 0/1
first = 1;                        % 1st image flag
second = 1;                       % 2nd image flag
sc = 0.4;                         % exploration amplitude - 0.4
N = 100;                          % Markov chain length - nb of particules
b = 0.2;                          % Markov burning ratio
mu = 200;                         % Dummy screwdriver - 20
W = [1/4 1/4 1/6 1/6 1/6];        % dimension weights for XYRGB
minWeight = 0;                    % particle minimum weight - 0.05
disp = 0;                         % display blue particles - 0/1
fo = fopen('Algo_Truth.txt','w'); % export algo truth (file out)

for i=FirstIm:LastIm
    file = sprintf('%s%0.4d.%s',nam,i,ext);
    path = sprintf('%s%s%s',nam,'/JPEGS/',file);
    im = imread(path); imrgb = im2double(im);
    
    if(first)      % 1st image
       f1 = figure(1); set(gcf,'Color',[0.2,0.2,0.2]);
       if ~debug, set(f1,'Position',[0 0 scr(3) scr(4)]);
       else set(f1,'Position',[2*scr(3)/3 scr(4)/2 scr(3)/3 scr(4)/2-100]);
       end
    end
    subplot(4,2,[1 3]); image(im); axis off; title(file,'color','w');

    if(first)      % 1st image
        first = 0;
        x = [43;67];    y = [194;224];  % WalkByShop1cor1    -  192

        % sample and draw the reference selection
        xr = x(1):cut:x(2); yr = y(1):cut:y(2);
        dispRect1([xr(1); xr(end)], [yr(1); yr(end)], 'y');

        % build the pdf reference model
        X = [xr(1); yr(1)];                         % upper left corner
        Height = length(yr); Width  = length(xr);   % grid dimensions
        
        %[Pr, ROIr, Ker, Map, KmR]
        [Pr, ~, Ker, Map, KmR] = buildModel(imrgb, X, Height, Width, ...
                                         cut, W, opt, Ker, K, Wker, Map);
                                       
        Np = size(Pr,1); % nb of points in the model
        fprintf('%s_%s_%d -> Np = %d\n', opt, dopt, cut, Np);

        % display luminance reference model
        dispMasterPicture(imrgb, X, Width, Height, cut);

        % fixed width & height magnitudes
        amph = sc*(((Height-1)*cut)+1);
        ampw = sc*(((Width -1)*cut)+1);

        % export values in a .txt file to compare with ground truth
        printValues(fo, i, X, Width, Height, cut);
    else
        if(second)      % 2nd image
            second = 0;
            Xchain = zeros(2,N); Wchain = zeros(1,N); Wtest = zeros(1,N);
            
            % last best particule
            %[Pt, ROIt, ~, ~, KmT] = ...
            [Pt, ~, ~, ~, KmT] = buildModel(imrgb, X, Height, Width, ...
                                           cut, W, opt, Ker, K, Wker, Map);
            %[w, Dkl] = ...
            w = calc_weight(mu, Pr, Pt, dopt, kNN, K, KmR, KmT, Wker);
            %disp_best_part(scr, ROIr, ROIt, Dkl, w);

            Dkl = zeros(N,1); % particules Dkl values
            parts = cell(1,N); % set of the particules
            p_feat = cell(5,1); % particules features
            
            % generation of the 1st Markov Chain
            for n = 1 : N
                % let draw a move & evaluate new proposal
                mh = round(2*amph.*(rand-0.5));
                mw = round(2*ampw.*(rand-0.5));

                % new particule
                Xp = X + [mw ; mh];
                if (disp), dispRect2(Xp, Width, Height, cut, '-b'); end;
                [Pt, ROIt, ~, ~, KmT] = buildModel(imrgb, Xp, Height, ...
                                    Width, cut, W, opt, Ker, K, Wker, Map);
                [wp, Dkl(n)] = calc_weight(mu, Pr, Pt, dopt, kNN, ...
                                                        K, KmR, KmT, Wker);
                Wtest(1,n) = wp;
                
                % select ramdomly according to likelihood ratio
                if min(1, wp/w) > rand
                    X = Xp;
                    w = wp;
                end
                Xchain(:,n) = X;
                Wchain(1,n) = w;
                
                % record particules features
                p_feat{1} = Xp; p_feat{2} = wp;   p_feat{3} = Dkl(n);
                p_feat{4} = Pt; p_feat{5} = ROIt; parts{n}  = p_feat;
            end
            
            % display sorted Dkl
            %[~, Idx] = sort(Dkl);
            %disp_part(scr, N, parts, Dkl, Idx, opt, cut, Np, dopt);
            
            % "burn" the chain
            Xchain = Xchain(:, fix(b*size(Xchain,2))+1 : end);
            Wchain = Wchain(   fix(b*length(Wchain))+1 : end);
            
        else       % all other images
            Xnew = zeros(2,N); Wnew = zeros(1,N); Wtest = zeros(1,N);

            % last best particule weight
            [Pt, ~, ~, ~, KmT] = buildModel(imrgb, X, Height, Width, ...
                                           cut, W, opt, Ker, K, Wker, Map);
            w = calc_weight(mu, Pr, Pt, dopt, kNN, K, KmR, KmT, Wker);
            
            %k=0;
            % Markov Chain
            for n = 1 : N
                % let's draw a move
                mh = round(2*amph.*(rand-0.5));
                mw = round(2*ampw.*(rand-0.5));

                % from a previous particule ramdomly selected
                ind = floor(length(Wchain) * rand + 1);
                
                Xp = Xchain(:,ind) + [mw ; mh];
                if (disp), dispRect2(Xp, Width, Height, cut, '-b'); end;
                
                [Pt, ~, ~, ~, KmT] = buildModel(imrgb, Xp, Height, ...
                                    Width, cut, W, opt, Ker, K, Wker, Map);
                wp = calc_weight(mu, Pr, Pt, dopt, kNN, K, KmR, KmT, Wker);
                Wtest(1,n) = wp;

                % select ramdomly according to likelihood ratio
                if min(1, wp/w) > rand
                    %k = k+1;
                    X = Xp;
                    w = wp;
                end
                Xnew(:,n) = X;
                Wnew(1,n) = w;
            end
            %fprintf('n°%d - tx = %0.1f%%\n', i, 100*k/N);

            % replace and cut the chain - cf. burn ratio : "b"
            Xchain = Xnew(:, fix(b*size(Xnew,2))+1 : end);
            Wchain = Wnew(   fix(b*length(Wnew))+1 : end);
        end

        % find the best current particule
        [maxW, idmax] = max(Wchain);
        X = Xchain(:, idmax);

        % display weight distribution
        dispWeightDist(Wtest, Wchain, mu, minWeight);

        if maxW^(1/mu) >= minWeight
            % if OK display intermediate particule
            figure(1); subplot(4,2,[1 3]);
            dispRect2(X, Width, Height, cut, '-y');
            dispInterPicture(imrgb, X, Width, Height, cut);

            % and integer it into the next search ?
        end

        % export values in a .txt file to compare with ground truth
        printValues(fo, i, X, Width, Height, cut);
    end
end
fclose(fo);

% fprintf('Press any key to continue...\n');
% pause; close all; clear all;
close all; clear all;

function [w, d] = calc_weight(mu, Pr, Pt, dopt, kNN, K, KmR, KmT, Wker)
% particule weight based on Kullback_Leibler divergence, SAD or SSD

if ~isempty(Pt)
    if     strcmp(dopt,'KLD')
        d = D_KL(Pr, Pt, kNN); % D_KL3(Pr, Pt, 3);
    elseif strcmp(dopt,'SAD')
        Dsad = abs(Pr-Pt);
        d = sum(Dsad(:));
    elseif strcmp(dopt,'SSD')
        Dssd = (Pr-Pt).^2;
        d = sum(Dssd(:));
    elseif strcmp(dopt,'KMI')
        MI = arrayfun(@(k) mut_inf__(KmR(:,:,k), KmT(:,:,k)), (1:K));
        d = - MI * Wker;
    else
        error('Wrong distance option');
    end
    
    w = exp(-mu*(d)); % return a weight
else
    w = 0;
end

function dispRect1(x, y, color)
d  = 0.5;
Px = [x(1)-d;x(2)+d;x(2)+d;x(1)-d;x(1)-d];
Py = [y(1)-d;y(1)-d;y(2)+d;y(2)+d;y(1)-d];
subplot(4,2,[1 3]);
hold on; plot(Px,Py,'color',color,'linewidth', 1); hold off;

function dispRect2(X, Width, Height, cut, color)
Px = [X(1); X(1)+(Width-1)*cut; X(1)+(Width-1)*cut; X(1); X(1)];
Py = [X(2); X(2); X(2)+(Height-1)*cut; X(2)+(Height-1)*cut; X(2)];
%figure(1); subplot(4,2,[1 3]);
hold on; plot(Px,Py,color,'linewidth', 1); hold off;

function dispMasterPicture(ImRGB, X, Width, Height, cut)
xr = X(1):cut:X(1)+(Width-1) *cut;
yr = X(2):cut:X(2)+(Height-1)*cut;

dy1 = yr(1) - 1;                         % distance from the top
dy2 = size(ImRGB,1) - yr(end);           % distance from the bottom
dx1 = xr(1) - 1;                         % distance from the left   
dx2 = size(ImRGB,2) - xr(end);           % distance from the right

if (dy1>=0)
    ystart = yr(1);
else
    ystart = yr(1) - dy1;
end
if (dy2>=0)
    yend = yr(end);
else
    yend = yr(end) + dy2;
end

if (dx1>=0)
    xstart = xr(1);
else
    xstart = xr(1) - dx1;
end
if (dx2>=0)
    xend = xr(end);
else
    xend = xr(end) + dx2;
end

yfull = ystart:yend; xfull = xstart:xend;

imCut = cat(3, ImRGB(yfull, xfull, 1), ...
               ImRGB(yfull, xfull, 2), ...
               ImRGB(yfull, xfull, 3));

figure(1); subplot(4,2,[2 4]); image(imCut);
title('Reference ROI','color','w'); axis image; axis off;

function dispInterPicture(ImRGB, X, Width, Height, cut)
xt = X(1):cut:X(1)+(Width-1) *cut;
yt = X(2):cut:X(2)+(Height-1)*cut;

dy1 = yt(1) - 1;                         % distance from the top
dy2 = size(ImRGB,1) - yt(end);           % distance from the bottom
dx1 = xt(1) - 1;                         % distance from the left   
dx2 = size(ImRGB,2) - xt(end);           % distance from the right

if (dy1>=0)
    ystart = yt(1);
else
    ystart = yt(1) - dy1;
end
if (dy2>=0)
    yend = yt(end);
else
    yend = yt(end) + dy2;
end

if (dx1>=0)
    xstart = xt(1);
else
    xstart = xt(1) - dx1;
end
if (dx2>=0)
    xend = xt(end);
else
    xend = xt(end) + dx2;
end

yfull = ystart:yend; xfull = xstart:xend;

imCut = cat(3, ImRGB(yfull, xfull, 1), ...
               ImRGB(yfull, xfull, 2), ...
               ImRGB(yfull, xfull, 3));

figure(1); subplot(4,2,[6 8]); image(imCut);
title('Best Target ROI','color','w'); axis image; axis off;

function dispWeightDist(Wtest, Wchain, mu, minWeight)
figure(1);

subplot(4,2,5);
%bar(sort(Wtest.^(1/mu), 'descend'), 'r');
bar(Wtest.^(1/mu), 'r');
set(gca,'Xlim',[0 length(Wtest)]);
set(gca,'Ylim',[0 1]);
title('Tested Weights','Color','w');

x = 0:0.1:length(Wtest);
hold on; plot(x, minWeight, 'r'); hold off;

subplot(4,2,7);
%bar(sort(Wchain.^(1/mu), 'descend'), 'b');
bar(Wchain.^(1/mu), 'b');
set(gca,'Xlim',[0 length(Wchain)]);
set(gca,'Ylim',[0 1]);
title('Selected Weights','Color','w');

x = 0:0.1:length(Wchain);
hold on; plot(x, minWeight, 'b'); hold off;

function [P, ROI, Ker, Map, Kmap] = buildModel(ImRGB, Xp, h, w, cut, ...
                                                 W, opt, Ker, K, Wker, Map)
Kmap = [];
if     strcmp(opt,'Raw') % uniform sampling & raw data
    [P, ROI] = buildModelRaw(ImRGB, Xp, h, w, cut, W);
    
elseif strcmp(opt,'Avg') % uniform sampling & average convolution
    [P, ROI] = buildModelAvg(ImRGB, Xp, h, w, cut, W);
    
elseif strcmp(opt,'Gauss') % uniform sampling & gaussian convolution
    [P, ROI] = buildModelGauss(ImRGB, Xp, h, w, cut, W);
    
elseif strcmp(opt,'RndRaw') % random sampling & raw data
    [P, ROI, Map] = buildModelRndRaw(ImRGB, Xp, h, w, cut, W, Map);
    
elseif strcmp(opt,'RndGauss') % random sampling & gaussian convolution
    [P, ROI, Map] = buildModelRndGauss(ImRGB, Xp, h, w, cut, W, Map);
    
elseif strcmp(opt,'RndVor')
    [P, ROI, Map] = buildModelRndVor(ImRGB, Xp, h, w, cut, W, Map);
    
elseif strcmp(opt,'KerOpt') % kernel selection sampling & convolution
    if isempty(Ker), Ker = learn_kernels_(K,'Opt'); end;
    [P, ROI, Kmap] = buildModelKer(ImRGB, Xp, h, w, cut, W, Ker, K, Wker);
else
    error('Unknown option');
end

function [P, ROI] = buildModelRaw(ImRGB, Xp, Height, Width, cut, W)
% Build a model from raw data only
[y x ystart yend xstart xend nstart nend mstart mend] = ...
    extractFullBounds(ImRGB, Xp, Height, Width, cut);

[Xin, Yin] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=Xin) & (Xin<=size(ImRGB,2)) & (1<=Yin) & (Yin<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(Xin,1), size(Xin,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

% Only full values if box out of the image
yfull = ystart:cut:yend; xfull = xstart:cut:xend;

[X, Y] = meshgrid(x, y);
P1 = (X-x(1))/(x(end)-x(1));
P2 = (Y-y(1))/(y(end)-y(1));

% pdf model - [0 1]^5
P1 = P1(nstart:nend, mstart:mend);           % X
P2 = P2(nstart:nend, mstart:mend);           % Y
P3 = ImRGB(yfull, xfull, 1);                 % R
P4 = ImRGB(yfull, xfull, 2);                 % G
P5 = ImRGB(yfull, xfull, 3);                 % B

P1 = P1(:); P2 = P2(:); P3 = P3(:); P4 = P4(:); P5 = P5(:);

if ~isempty(P5)
    P = cat(2, W(1)*P1, W(2)*P2, W(3)*P3, W(4)*P4, W(5)*P5);
else
    P = [];
end

function [P, ROI] = buildModelAvg(ImRGB, Xp, Height, Width, cut, W)
% Build a model from data smoothed by an average kernel
[y x ystart yend xstart xend nstart nend mstart mend] = ...
    extractFullBounds(ImRGB, Xp, Height, Width, cut);

[Xin, Yin] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=Xin) & (Xin<=size(ImRGB,2)) & (1<=Yin) & (Yin<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(Xin,1), size(Xin,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

% Only full values if box out of the image
yfull = ystart:cut:yend; xfull = xstart:cut:xend;

ImR2 = filter2(fspecial('average',[cut cut]), ImRGB(:,:,1));
ImG2 = filter2(fspecial('average',[cut cut]), ImRGB(:,:,2));
ImB2 = filter2(fspecial('average',[cut cut]), ImRGB(:,:,3));

[X, Y] = meshgrid(x, y);
P1 = (X-x(1))/(x(end)-x(1));
P2 = (Y-y(1))/(y(end)-y(1));

% pdf model - [0 1]^5
P1 = P1(nstart:nend, mstart:mend);           % X
P2 = P2(nstart:nend, mstart:mend);           % Y
P3 = ImR2(yfull, xfull);                     % R smooth
P4 = ImG2(yfull, xfull);                     % G smooth
P5 = ImB2(yfull, xfull);                     % B smooth

P1 = P1(:); P2 = P2(:); P3 = P3(:); P4 = P4(:); P5 = P5(:);

if ~isempty(P5)
    P = cat(2, W(1)*P1, W(2)*P2, W(3)*P3, W(4)*P4, W(5)*P5);
else
    P = [];
end

function [P, ROI] = buildModelGauss(ImRGB, Xp, Height, Width, cut, W)
% Build a model from data smoothed by a gaussian kernel
[y x ystart yend xstart xend nstart nend mstart mend] = ...
    extractFullBounds(ImRGB, Xp, Height, Width, cut);

[Xin, Yin] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=Xin) & (Xin<=size(ImRGB,2)) & (1<=Yin) & (Yin<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(Xin,1), size(Xin,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

% Only full values if box out of the image
yfull = ystart:cut:yend; xfull = xstart:cut:xend;

sigma = 1; % 1: soft smoothing / 0.5: hard smoothing
ImR2 = filter2(fspecial('gaussian',[cut cut],sigma), ImRGB(:,:,1));
ImG2 = filter2(fspecial('gaussian',[cut cut],sigma), ImRGB(:,:,2));
ImB2 = filter2(fspecial('gaussian',[cut cut],sigma), ImRGB(:,:,3));

[X, Y] = meshgrid(x, y);
P1 = (X-x(1))/(x(end)-x(1));
P2 = (Y-y(1))/(y(end)-y(1));

% pdf model - [0 1]^5
P1 = P1(nstart:nend, mstart:mend);           % X
P2 = P2(nstart:nend, mstart:mend);           % Y
P3 = ImR2(yfull, xfull);                     % R smooth
P4 = ImG2(yfull, xfull);                     % G smooth
P5 = ImB2(yfull, xfull);                     % B smooth

P1 = P1(:); P2 = P2(:); P3 = P3(:); P4 = P4(:); P5 = P5(:);

if ~isempty(P5)
    P = cat(2, W(1)*P1, W(2)*P2, W(3)*P3, W(4)*P4, W(5)*P5);
else
    P = [];
end

function [P, ROI, Map] = buildModelRndRaw(ImRGB, Xp, Height, Width, ...
                                                         cut, W, Map)
% Build a model from random raw data
[y x ystart yend xstart xend]= extractBounds(ImRGB, Xp, Height, Width,cut);

[X, Y] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=X) & (X<=size(ImRGB,2)) & (1<=Y) & (Y<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(X,1), size(X,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

RX = (X-x(1))/(x(end)-x(1));
RY = (Y-y(1))/(y(end)-y(1));
ImR = ROI(:,:,1);
ImG = ROI(:,:,2);
ImB = ROI(:,:,3);

% Choose randomly Np pixels among the ROI
if isempty(Map)
    yfull = ystart:cut:yend; xfull = xstart:cut:xend;
    Np = length(yfull)*length(xfull);
    Map = round((numel(X)-1)*rand(Np,1)) + 1;
end

% keep values in column
RX = RX(:); RY = RY(:); ImR = ImR(:); ImG = ImG(:); ImB = ImB(:);

% pdf model - [0 1]^5
P1 = RX(Map(In(Map)));     % X
P2 = RY(Map(In(Map)));     % Y
P3 = ImR(Map(In(Map)));    % R
P4 = ImG(Map(In(Map)));    % G
P5 = ImB(Map(In(Map)));    % B

if ~isempty(P5)
    P = cat(2, W(1)*P1, W(2)*P2, W(3)*P3, W(4)*P4, W(5)*P5);
else
    P = [];
end

function [P, ROI, Map] = buildModelRndGauss(ImRGB, Xp, Height, Width, ...
                                                         cut, W, Map)
% Build a model from random data smoothed by a gaussian kernel
[y x ystart yend xstart xend]= extractBounds(ImRGB, Xp, Height, Width,cut);

[X, Y] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=X) & (X<=size(ImRGB,2)) & (1<=Y) & (Y<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(X,1), size(X,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;  % raw ROI

sigma = 1; % 1 : soft smoothing - 0.5 : hard smoothing
Rs = filter2(fspecial('gaussian',[cut cut],sigma), ROI1(:,:,1));
Gs = filter2(fspecial('gaussian',[cut cut],sigma), ROI1(:,:,2));
Bs = filter2(fspecial('gaussian',[cut cut],sigma), ROI1(:,:,3));
ROIs = cat(3, Rs, Gs, Bs);

ROI2 = zeros(size(X,1), size(X,2), 3);
ROI2(repmat(In,[1 1 3])) = ROIs; % smoothed ROI

RX = (X-x(1))/(x(end)-x(1));
RY = (Y-y(1))/(y(end)-y(1));
ImR = ROI2(:,:,1);
ImG = ROI2(:,:,2);
ImB = ROI2(:,:,3);

% Choose randomly Np pixels among the ROI
if isempty(Map)
    yfull = ystart:cut:yend; xfull = xstart:cut:xend;
    Np = length(yfull)*length(xfull);
    Map = round((numel(X)-1)*rand(Np,1)) + 1;
end

% keep values in column
RX = RX(:); RY = RY(:); ImR = ImR(:); ImG = ImG(:); ImB = ImB(:);

% pdf model - [0 1]^5
P1 = RX(Map(In(Map)));     % X
P2 = RY(Map(In(Map)));     % Y
P3 = ImR(Map(In(Map)));    % R smooth
P4 = ImG(Map(In(Map)));    % G smooth
P5 = ImB(Map(In(Map)));    % B smooth

if ~isempty(P5)
    P = cat(2, W(1)*P1, W(2)*P2, W(3)*P3, W(4)*P4, W(5)*P5);
else
    P = [];
end

function [P, ROI, Map] = buildModelRndVor(ImRGB, Xp, Height, Width, ...
                                                         cut, W, Map)
% Build a model from random raw data
[y x ystart yend xstart xend]= extractBounds(ImRGB, Xp, Height, Width,cut);

[X, Y] = meshgrid(x(1):x(end), y(1):y(end));
In = (1<=X) & (X<=size(ImRGB,2)) & (1<=Y) & (Y<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(X,1), size(X,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

RX = (X-x(1))/(x(end)-x(1));
RY = (Y-y(1))/(y(end)-y(1));
ImR = ROI(:,:,1);
ImG = ROI(:,:,2);
ImB = ROI(:,:,3);

% Choose randomly Np pixels among the ROI
if isempty(Map)
    yfull = ystart:cut:yend; xfull = xstart:cut:xend;
    Np = length(yfull)*length(xfull); % nb of sites
    Idx = round((numel(X)-1)*rand(Np,1)) + 1;
    
    sx = X(Idx); sy = Y(Idx); % sites position
    D2 = sq_dist([sx(:),sy(:)],[X(:),Y(:)]);
    [~, lab] = min(D2, [], 1);
    Map = reshape(lab, size(X)); % labels map
end

Kc = 0;
K = max(Map(:)); % nb of cells a priori
P = zeros(K,5);  % XYRGB

for k = 1 : K
    cell = (Map == k) & In;
    % figure; image(cell); colormap(gray(2)); axis image;
    
    if ~isempty(In(cell))
        Kc = Kc + 1;
        
        % pdf model - [0 1]^5 - mean value in each cell
        u = mean(RX(cell));                % U
        v = mean(RY(cell));                % V
        r = mean(ImR(cell));               % R
        g = mean(ImG(cell));               % G
        b = mean(ImB(cell));               % B

        P(k,:) = [W(1)*u, W(2)*v, W(3)*r, W(4)*g, W(5)*b];
    end
end

P = P(1:Kc,:);

function [P, ROI, Kmap] = buildModelKer(ImRGB, Xp, Height, Width, cut, ...
                                                           W, Ker, K, Wker)
[y x ystart yend xstart xend]=extractBounds(ImRGB, Xp, Height, Width, cut);

[X, Y] = meshgrid(x, y);
In = (1<=X) & (X<=size(ImRGB,2)) & (1<=Y) & (Y<=size(ImRGB,1));
ROI1 = ImRGB(ystart:yend, xstart:xend, :);
ROI = zeros(size(X,1), size(X,2), 3);
ROI(repmat(In,[1 1 3])) = ROI1;

RX = (X-x(1))/(x(end)-x(1));
RY = (Y-y(1))/(y(end)-y(1));

% scr = get(0,'ScreenSize');
% f3 = figure(3); set(gcf,'Color',[0.2,0.2,0.2]);
% set(f3,'Position', [scr(3)/3 0 scr(3)/3 scr(4)/2-50*2]);
% L = floor((K-1)/5)+1;

P = zeros(K,5); % XYRGB
Kmap = zeros(Height, Width, K);
for k = 1:K
    Pk = pmap_wgt(Ker{k}{4}, ROI, Ker{k}{1}(1), Ker{k}{1}(2), ...
                                                     Ker{k}{3}, Ker{k}{7});
    % [Pk, Wk] = ...
    % subplot(L,5,k); image(255*Pk);
    % colormap(gray(256)); axis image; axis off;
    
    Kmap(:,:,k) = Pk;
    
    Kf = Pk/sum(Pk(:)); % normalized kernel filter
    
    % pdf model - [0 1]^5
    X1 = Kf .* RX;                           % X
    X2 = Kf .* RY;                           % Y
    X3 = Kf .* ROI(:,:,1);                   % R
    X4 = Kf .* ROI(:,:,2);                   % G
    X5 = Kf .* ROI(:,:,3);                   % B

    % mean value of the pixel, ROI reduced by the kernel reduction
    x=sum(X1(:)); y=sum(X2(:)); r=sum(X3(:)); g=sum(X4(:)); b=sum(X5(:));
    
    % P(k,:) = [W(1)*x, W(2)*y, W(3)*r, W(4)*g, W(5)*b];
    
    P(k,:) = [W(1)*x, W(2)*y, W(3)*r, W(4)*g, W(5)*b]*Wker(k)*K;
    
%     if     strcmp(kopt, 'Wnorm')
%         P(k,:) = [W(1)*x, W(2)*y, W(3)*r, W(4)*g, W(5)*b];
%         
%     elseif strcmp(kopt, 'Wdim')
%         P(k,:) = [Wk(1)*x, Wk(2)*y, Wk(3)*r, Wk(4)*g, Wk(5)*b];
%         
%     elseif strcmp(kopt, 'Wker')
%         P(k,:) = [W(1)*x, W(2)*y, W(3)*r, W(4)*g, W(5)*b] * ...
%                                   (Ker{k}{6}*K);
%         
%     elseif strcmp(kopt, 'Wboth')
%         P(k,:) = [Wk(1)*x, Wk(2)*y, Wk(3)*r, Wk(4)*g, Wk(5)*b] * ...
%                                   (Ker{k}{6}*K);
%     end
end

function [y x ystart yend xstart xend nstart nend mstart mend] = ...
    extractFullBounds(ImRGB, X, Height, Width, cut)
%return correct bounds if box not completely inside image
x = X(1):cut:X(1)+(Width-1) *cut;
y = X(2):cut:X(2)+(Height-1)*cut;

dy1 = y(1) - 1;                         % distance from the top
dy2 = size(ImRGB,1) - y(end);           % distance from the bottom
dx1 = x(1) - 1;                         % distance from the left   
dx2 = size(ImRGB,2) - x(end);           % distance from the right

if (dy1>=0)
    ystart = y(1);
    nstart = 1;
else
    ystart = y(1) - dy1;
    nstart = ceil(-dy1/cut) + 1;
end
if (dy2>=0)
    yend = y(end);
    nend = Height;
else
    yend = y(end) + dy2;
    nend = Height - ceil(-dy2/cut);
end

if (dx1>=0)
    xstart = x(1);
    mstart = 1;
else
    xstart = x(1) - dx1;
    mstart = ceil(-dx1/cut) + 1;
end
if (dx2>=0)
    xend = x(end);
    mend = Width;
else
    xend = x(end) + dx2;
    mend = Width - ceil(-dx2/cut);
end

function [y x ystart yend xstart xend] = ...
    extractBounds(ImRGB, X, Height, Width, cut)
%return correct bounds if box not completely inside image
x = X(1):cut:X(1)+(Width-1) *cut;
y = X(2):cut:X(2)+(Height-1)*cut;

dy1 = y(1) - 1;                         % distance from the top
dy2 = size(ImRGB,1) - y(end);           % distance from the bottom
dx1 = x(1) - 1;                         % distance from the left   
dx2 = size(ImRGB,2) - x(end);           % distance from the right

if (dy1>=0)
    ystart = y(1);
else
    ystart = y(1) - dy1;
end
if (dy2>=0)
    yend = y(end);
else
    yend = y(end) + dy2;
end

if (dx1>=0)
    xstart = x(1);
else
    xstart = x(1) - dx1;
end
if (dx2>=0)
    xend = x(end);
else
    xend = x(end) + dx2;
end

function printValues(fid, i, X, Width, Height, cut)
xc = X(1)+round((Width-1)*cut/2);
yc = X(2)+round((Height-1)*cut/2);

fprintf(fid, 'Image n°;%1.0f; ', i);
fprintf(fid, '- ROI : ');
fprintf(fid, 'x = [;%1.0f;%1.0f;] ', X(1), X(1)+(Width-1)*cut);
fprintf(fid, 'y = [;%1.0f;%1.0f;] ', X(2), X(2)+(Height-1)*cut);
fprintf(fid, '- Width = ;%1.0f; ', Width);
fprintf(fid, ' Height = ;%1.0f; ', Height);
fprintf(fid, ' cut = ;%1.0f; ', cut);
fprintf(fid, '- xc = ;%1.0f; ', xc);
fprintf(fid, ' yc = ;%1.0f\n', yc);

function [P, W] = pmap_wgt(lambda, ImRGB, i, j, dim, label)
% kernel probability map and weight vector in XYRGB
[h,w,~] = size(ImRGB);
[X,Y] = meshgrid(1:w,1:h);

if strcmp(dim,'R')
    R = ImRGB(:,:,1); r = ImRGB(i,j,1); D = (R-r).^2;
    W = [0 0 1 0 0];
elseif strcmp(dim,'G')
    G = ImRGB(:,:,2); g = ImRGB(i,j,2); D = (G-g).^2;
    W = [0 0 0 1 0];
elseif strcmp(dim,'B')
    B = ImRGB(:,:,3); b = ImRGB(i,j,3); D = (B-b).^2;
    W = [0 0 0 0 1];
elseif strcmp(dim,'XYR')
    Dxy = 1/2*(((X-j)./(w-1)).^2+((Y-i)./(h-1)).^2);
    R = ImRGB(:,:,1); r = ImRGB(i,j,1); Dr = (R-r).^2;
    D = 1/2*Dxy + 1/2*Dr;
    W = [0.25 0.25 0.5 0 0];
elseif strcmp(dim,'XYG')
    Dxy = 1/2*(((X-j)./(w-1)).^2+((Y-i)./(h-1)).^2);
    G = ImRGB(:,:,2); g = ImRGB(i,j,2); Dg = (G-g).^2;
    D = 1/2*Dxy + 1/2*Dg;
    W = [0.25 0.25 0 0.5 0];
elseif strcmp(dim,'XYB')
    Dxy = 1/2*(((X-j)./(w-1)).^2+((Y-i)./(h-1)).^2);
    B = ImRGB(:,:,3); b = ImRGB(i,j,3); Db = (B-b).^2;
    D = 1/2*Dxy + 1/2*Db;
    W = [0.25 0.25 0 0 0.5];
elseif strcmp(dim,'XY')
    D = 1/2*(((X-j)./(w-1)).^2+((Y-i)./(h-1)).^2);
    W = [0.5 0.5 0 0 0];
elseif strcmp(dim,'RGB')
    R = ImRGB(:,:,1); r = ImRGB(i,j,1); Dr = (R-r).^2;
    G = ImRGB(:,:,2); g = ImRGB(i,j,2); Dg = (G-g).^2;
    B = ImRGB(:,:,3); b = ImRGB(i,j,3); Db = (B-b).^2;
    D = 1/3*Dr + 1/3*Dg + 1/3*Db;
    W = [0 0 1/3 1/3 1/3];
elseif strcmp(dim,'XYRGB')
    Dxy = 1/2*(((X-j)./(w-1)).^2+((Y-i)./(h-1)).^2);
    R = ImRGB(:,:,1); r = ImRGB(i,j,1); Dr = (R-r).^2;
    G = ImRGB(:,:,2); g = ImRGB(i,j,2); Dg = (G-g).^2;
    B = ImRGB(:,:,3); b = ImRGB(i,j,3); Db = (B-b).^2;
    Drgb = 1/3*Dr + 1/3*Dg + 1/3*Db;
    D = 1/2*Dxy + 1/2*Drgb;
    W = [1/4 1/4 1/6 1/6 1/6];
else
    error('Unknown kernel dimension option');
end

% calculate the probability map
if     label == 1            % xm belongs to foreground
    P = exp(-lambda*D);
elseif label == 0            % xm belongs to background
    P = 1 - exp(-lambda*D);
else
    error('Kernel has no label assigned');
end

function D2 = sq_dist(X,Y)   % square distances
nx = size(X,1); ny = size(Y,1);
D2 = sum(X.^2,2)*ones(1,ny) + ones(nx,1)*sum(Y.^2,2)' - 2*(X*Y');
 
% function disp_best_part(scr, ROIr, ROIt, Dkl, w)
% f3 = figure(3); set(gcf,'Color',[0.2,0.2,0.2]);
% set(f3,'Position', [scr(3)/3, scr(4)/2, scr(3)/3, scr(4)/2-50*2]);
% 
% tit = sprintf('Dkl = %0.3f - w = %0.3f',Dkl, w);
% subplot(1,2,1); image(ROIr); axis image; axis off;
% subplot(1,2,2); image(ROIt); axis image; axis off;title(tit,'color','w');

% function disp_part(scr, N, parts, Dkl, Idx, opt, cut, Np, dopt)
% % display sorted particules
% f4 = figure(4); set(gcf,'Color',[0.2,0.2,0.2]);
% set(f4,'Position', [0 scr(4)/2 scr(3)/3 scr(4)-50*2]);
%             
% for n = 1 : N
%     ROIt = parts{Idx(n)}{5};
%     Dkl_n  = Dkl(Idx(n));
%     
%     subplot(10,10,n); image(ROIt); axis image; axis off;
%     tit = sprintf('%0.3f',Dkl_n); title(tit,'color','w');
% end
% 
% f5 = figure(5); set(gcf,'Color',[0.2,0.2,0.2]);
% set(f5,'Position', [scr(3)/3, 0, scr(3)/3, scr(4)/2-50*2]);
% tit = sprintf('Distances triées - %s - %s - cut = %d -> Np = %d', ...
%                                                      opt, dopt, cut, Np);
% bar(Dkl(Idx)); title(tit,'color','w'); xlim([0 N]);

function Wker = disp_kernels(scr, Ker, K)
% display selected kernels
f2 = figure(2); set(gcf,'Color',[0.2,0.2,0.2]);
set(f2,'Position', [2*scr(3)/3 0 scr(3)/3 scr(4)/2-50*2]);

L = floor((K-1)/5)+1; % nb of rows
Wk = zeros(K,1);      % vector of absolute weights

for k=1:K
    Wk(k) = Ker{k}{5};
    tit = sprintf('[%2d,%2d] %s',Ker{k}{1}(1),Ker{k}{1}(2),Ker{k}{3});
    subplot(L,5,k); image(uint8(255*Ker{k}{8})); colormap(gray(256));
    title(tit,'Color','w'); axis image; axis off;
end

 Wker = Wk/sum(Wk); % provide relative weights
 
%******************************** Entropy ********************************%

function H2 = Hp(P)
% Entropy of a state of n micro-states
% P: micro-states probabilities vector (whatever in line or in column)
if abs(sum(P)-1) > 1000*eps
    error('Sum of probabilities not equal to 1: %0.4f', sum(P));
end
Ok = (P > eps) & (P < 1-eps);     % avoids NaN as p.log(p) -> 0 in 0+
H2 = sum(P(Ok).*log2(1./P(Ok)));  % normalization in bits

function H = H1p(P)
% Entropy of a map of probabilities
% P : probability map of some event - ]0,1[
p = mean(P(:));
H = Hp([p,1-p]);

function H = H2pp(P,Q)
% Joint Entropy of 2 probability maps
% P,Q : probability maps of an event - [0,1]
P = P(:); Q = Q(:); n = length(P);

P2 = zeros(n, 2^2);
S = arrayfun(@(k) square(P(k),Q(k)),(1:n)','UniformOutput',0);
for k=1:n, P2(k,:) = S{k}; end;

H = Hp(mean(P2,1));

function p2 = square(px, py)
% Find the set of probabilities [p00,p01,p10,p11]
% matching to px and py so that flip-flops are minimized
[p00,id1] = min([1-px,1-py]); p11 = min([px,py]);

if id1 == 1
    p2 = [p00, 0, 1-p00-p11, p11]; % p01 = 0;
else
    p2 = [p00, 1-p00-p11, 0, p11]; % p10 = 0;
end

function I = mut_inf__(P,Q)
% Mutual Information between 2 probability maps
I = H1p(P) + H1p(Q) - H2pp(P,Q);
