% Run batch tests - 10/15/11
% Jean-Marc Berthommé
%
% - 10/15/11:
%   . file to run batch tests on different configurations
%   . manage XYRGB_knn_tracker comments to run 'run'

function run
cell_opt = {'Raw','Avg','Gauss','RndRaw','RndGauss','RndVor','KerOpt'};
cell_dist = {'KLD','SAD','SSD','KMI'};
cell_K = {5, 12, 20, 56, 100};

FirstIm = 192;
LastIm  = 309;
FileIn = 'Algo_Truth.txt';
K = 0;

for i_opt = 1:3 % 1:3 / 6:7
  opt = cell_opt{i_opt};
    
  for i_dist = 1:1 % KLD only
    dopt = cell_dist{i_dist};
    
    for cut = 4:3:10 % 1:10
      for i = 1:10   % trial number
        
        % call the program with the proper options
        XYRGB_knn_tracker(FirstIm, LastIm, cut, opt, dopt, K);

        % Read -> Copy
        fi = fopen(FileIn,'r');
        AT = importValues(fi, FirstIm, LastIm); % Algorithm Truth

        % Write -> Paste
        FileOut = sprintf('Analyse/%s_%s_%d_v%d.txt', opt, dopt, cut, i);
        fo = fopen(FileOut,'w');
        exportValues(fo, AT, FirstIm, LastIm);
      end
    end
  end
end

cut = 1;
for i_opt = 7:7 % KerOpt
  opt = cell_opt{i_opt};
  
  for i_dist = 1:1 % KLD
    dopt = cell_dist{i_dist};
    
    for i_K = 1:5
      K = cell_K{i_K};
      for i=1:10 % trial number
        
        % call the program with the proper options
        XYRGB_knn_tracker(FirstIm, LastIm, cut, opt, dopt, K);

        % Read -> Copy
        fi = fopen(FileIn,'r');
        AT = importValues(fi, FirstIm, LastIm); % Algorithm Truth

        % Write -> Paste
        FileOut = sprintf('Analyse/%s_%s_%d_v%d.txt', opt, dopt, K, i);
        fo = fopen(FileOut,'w');
        exportValues(fo, AT, FirstIm, LastIm);
      end
    end
  end
end

function GT = importValues(fi, FirstImage, LastImage)
GT = [];
for i=FirstImage:LastImage
    [nIm x1 x2 y1 y2 Width Height cut xc yc] = getValues(fi);
    gt = [nIm x1 x2 y1 y2 Width Height cut xc yc];
    GT = cat(1, GT, gt);
end
fclose(fi);

function [nIm x1 x2 y1 y2 Width Height cut xc yc] = getValues(fid)
fread(fid, 9, '*char');
nIm = fscanf(fid, '%d');
fread(fid, 16, '*char');
x1 = fscanf(fid, '%d');
fread(fid, 1, '*char');
x2 = fscanf(fid, '%d');
fread(fid, 9, '*char');
y1 = fscanf(fid, '%d');
fread(fid, 1, '*char');
y2 = fscanf(fid, '%d');
fread(fid, 14, '*char');
Width = fscanf(fid, '%d');
fread(fid, 13, '*char');
Height = fscanf(fid, '%d');
fread(fid, 10, '*char');
cut = fscanf(fid, '%d');
fread(fid, 10, '*char');
xc = fscanf(fid, '%d');
fread(fid, 9, '*char');
yc = fscanf(fid, '%d');

function exportValues(fo, GT, FirstImage, LastImage)

for i=FirstImage:LastImage
    k = i - FirstImage + 1;
    printValues(fo, i, [GT(k,2), GT(k,4)], GT(k,6), GT(k,7), GT(k,8));
end
fclose(fo);

function printValues(fid, i, X, Width, Height, cut)
xc = X(1)+round((Width-1)*cut/2);
yc = X(2)+round((Height-1)*cut/2);

fprintf(fid, 'Image n°;%1.0f; ', i);
fprintf(fid, '- ROI : ');
fprintf(fid, 'x = [;%1.0f;%1.0f;] ', X(1), X(1)+(Width-1)*cut);
fprintf(fid, 'y = [;%1.0f;%1.0f;] ', X(2), X(2)+(Height-1)*cut);
fprintf(fid, '- Width = ;%1.0f; ', Width);
fprintf(fid, ' Height = ;%1.0f; ', Height);
fprintf(fid, ' cut = ;%1.0f; ', cut);
fprintf(fid, '- xc = ;%1.0f; ', xc);
fprintf(fid, ' yc = ;%1.0f\n', yc);